(* 
   File   : /home/cvs/cvsroot/projects/visualstate/scope/HsmAbsyn.sml,v
   Comment: Convert Hsm ASTs to Absyn ASTs
   Release: scope-0-15_final

     Copyright (C) 2003 Andrzej Wasowski
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
     
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
     
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

   HsmAbsyn.sml,v 1.2 2003/07/08 15:45:01 wasowski Exp
*)

structure HsmAbsyn : CONVIN = struct

open Absyn Say Err

structure H  = Hsm
structure S  = Symbol
structure SE = SymbolEnv

type source = H.hsm * string (* model, fname *)
type aor    = unit
type atr    = unit

fun unzip3 [] = ([],[],[])
  | unzip3 ((a,b,c)::ta) = let val (x,y,z) = unzip3 ta
			   in (a::x,b::y,c::z) end

fun conv_ac action = Expr (Call (S.symbol action,[]))

fun conv_tr scope {event,action,source,target} =
	{ 
		trigger = Event (S.symbol event,[]),
		scope   = S.symbol scope,
		pos     = [S.symbol (String.concat 
				(Typeset.join_dots (scope::source) []))],
		neg     = nil,
		guard   = [],
		action  = (map conv_ac action,[]),
		targets = [S.symbol (String.concat
				(Typeset.join_dots (scope::target) []))],
		annotation= ()

	}

		

fun conv_hand pid (H.AND (id,initial,ors),(siblings,trans,denv)) 
		:aor andstate list * atr tran list * denv = 
	let val sid  = case pid of NONE => id
				 | SOME pid => pid ^ "." ^ id
	    val (kids,trans1,denv1) = foldr (conv_hor sid) ([],trans,denv) ors
	    val this =  ANDState { name    = S.symbol sid,
				   parent  = Option.map (fn s=> S.symbol s) pid,   
				   children= kids,
				   entry   = ([],[]),
				   exit    = ([],[]) }
	    val denv2 = if not initial then denv1 
			else SE.bind1 denv1 (S.symbol (valOf pid), S.symbol sid)
			handle Option.Option => denv1
	in (this::siblings, trans1, denv2) end 
	handle e => intErr e 200305077

and conv_hor pid (H.OR (id,ands,ltrans),(siblings,trans,denv))
		:aor orstate list * atr tran list * denv =
	let val sid = pid ^ "." ^ id
	    val (kids,trans1,denv1) = foldr (conv_hand (SOME sid)) 
					([],trans,denv) ands
	    val this = ORState { name = S.symbol sid, parent  = S.symbol pid, 
				 children= kids, marking = Initial,
				 annotation= () }
	    val trans2 = map (conv_tr sid) ltrans @ trans1
	in (this::siblings,trans2,denv1) end 
	handle e => intErr e 200305076

fun conv_sys (fname :string) {events, actions, 
			      hierarchy=h as H.AND(id,initial,ors)} =
	let val (h,trans,denv) = conv_hand NONE (h,([],[],SE.empty))
	in { 
		name     = S.symbol id,
		fname    = S.symbol fname,
		hierarchy= idx_hierarchy (hd h),
		vartypes = SE.empty,
		evtypes  = SE.fromList(map (fn e=>(S.symbol e,(false,[]))) 
					events),
		evgroups = SE.empty,
		otypes   = SE.fromList(map (fn a=>(S.symbol a,(VoidT,[],false))) 
					actions),
		denv     = denv,
		venv     = SE.empty,
		cenv     = SE.empty,
		trans    = trans }
	 end handle e => intErr e 200305075

	

fun translate (hsm :H.hsm, fname :string) :(aor,atr) model = [conv_sys fname hsm]


end

